#Using Inventek IWIN AT Commands, this Python script will:

#1. Detect Comm port
#2. Join a network that you edit below
#	rcv = sendCMDBlocking(ser, "C1=xxxx")                #Set SSID
#        rcv = sendCMDBlocking(ser, "C2=ThisIsMyNetwork1")   #Set Password
#        rcv = sendCMDBlocking(ser, "C3=4")                  #WPA-2 Mixed
#        rcv = sendCMDBlocking(ser, "C4=1")                  #Use DHCP with detect the comm port and do a HTTP Get to a web page using DNS lookup
#3. Do a DNS web site lookup using D0 and polulate the P3 AT command 
#4. HTTP web page Get request 
#5. Read some of the web page
#6  Close connection
#-------------------------------------------------------------------------------

import math, sys, serial, serial.tools.list_ports, time

#Quiet mode, 0 = Quiet, 1 = Verbose
quiet = 1

#Send timeout
timeout = 30


#Send a command to a eS-WiFi module and wait for prompt("\r\n> ") responce
def sendCMDBlocking(serPort,cmd):
    rcvString =""

    #Send Command
    if (quiet == 1):
        sys.stdout.write(cmd)
    serPort.write(cmd + '\r')

    st = time.time()                    #Get time

    #read until "\r\n> " received
    found = 0
    while found != 1:
        for c in serPort.read():
            rcvString += c
            if  '\r\n> ' in rcvString:
                found = 1
            #Endif
        #End for

        #Check for timeout
        if (time.time() - st) > timeout:
            return '-1'                 #Timeout error
        #Endif
    #End while

    #Show responce
    if (quiet == 1):
        sys.stdout.write(rcvString)

    #Get payload if OK responce
    if 'OK\r\n> ' in rcvString:
        rcvString=rcvString[2:(len(rcvString)-8)]
    #Endif

    #Return payload or Error
    return rcvString


def sendDataBlocking(serPort,data):
    rcvString =""

    #Send Command
    if (quiet == 1):
        sys.stdout.write(data)
    serPort.write(data)

    st = time.time()                    #Get time

    #read until "\r\n> " received
    found = 0
    while found != 1:
        for c in serPort.read():
            rcvString += c
            if  '\r\n> ' in rcvString:
                found = 1
            #Endif
        #End for

        #Check for timeout
        if (time.time() - st) > timeout:
            return '-1'                 #Timeout error
        #Endif
    #End while

    #Show responce
    if (quiet == 1):
        sys.stdout.write(rcvString)

    #Get payload if OK responce
    if 'OK\r\n> ' in rcvString:
        rcvString=rcvString[2:(len(rcvString)-8)]
    #Endif

    #Return payload or Error
    return rcvString

#Send CMD and wait for responce, waiting nsec seconds before timing out
def sendCMDWait(serPort,cmd,responce,nsec):
    rcvString =""

    #Send Command
    if (quiet == 1):
        sys.stdout.write(cmd)
    serPort.write(cmd + '\r')

    st = time.time()                    #Get time

    #Check each message recieved for responce
    found = 0
    while found != 1:

        #read until "\r\n> " received
        found0 = 0
        while found0 != 1:
            for c in serPort.read():
                rcvString += c
                #if (quiet == 1):
                sys.stdout.write(c)
                if  "\r\n> " in rcvString:
                    found0 = 1
                #Endif
            #End for

            #Check for timeout
            if (time.time() - st) > nsec:
                return '-1'                 #Timeout error
            #Endif
        #End while

        #Show responce
        if (quiet == 1):
            sys.stdout.write(rcvString)

        #Check for responce
        if  responce in rcvString:
            found = 1
        else:
            rcvString = ""                  #Clear string
        #End if
    #End while

    #Show responce
    if (quiet == 1):
        sys.stdout.write(rcvString)

    #Get payload if OK responce
    if 'OK\r\n> ' in rcvString:
        rcvString=rcvString[2:(len(rcvString)-8)]
    #Endif

    #Return payload or Error
    return rcvString


#Search COM port for and eS-WiFi module
def findModule(timeout):
    rcvString =""

    #Get ports
    portList = list(serial.tools.list_ports.comports())
    #print(portList)

    #Search for module
    for index, openPort in enumerate(portList):

        #Serial Port Setup
        port = portList[index][0]
        baud = 115200
        serPort = serial.Serial(port,baud)
        serPort.timeout = 10

        serPort.write('\r')                 #Send CR

        st = time.time()                    #Get time

        #read until "\r\n> " received
        found = 0
        while found != 1:
            for c in serPort.read():
                rcvString += c
                if  '\r\n> ' in rcvString:
                    found = 1
                #Endif
            #End for

            #Check for timeout
            if (time.time() - st) > timeout:
                return '-1'                 #Timeout error
            #Endif
        #End while

        #Try to get info
        rcv = sendCMDBlocking(serPort,"I?")

        #Close serial port
        serPort.close()

        #Found?
        if (found == 1):
            #print("Port = " + port)
            if "ISM4" in rcv:
                print(rcv)
                return port
            #Endif
        #End if
    #End for
    return -1

#Application
def main():

    #Redirect std err to std out
    sys.stderr = sys.stdout

    #Serial Port Setup
    port = findModule(10)
    if (port == -1):
        print( "Failed to find an eS-WiFi module!")
        sys.exit(-1)
    #End if
    baud = 115200
    ser = serial.Serial(port,baud)
    ser.timeout = 10

    bd = 115200                     #Default
    #Choose baud rate for demo (uncomment choice)
    bd = 115200
    #bd = 921600
    #bd = 1843200
    #bd = 2073600
    #bd = 3916800

    #Get Firmware information
    if (quiet == 1):
        rcv = sendCMDBlocking(ser,"I?")
        print(rcv)
    #End if

    #Setup
    mCnt = 1 #910                  #Monitoring Count
    #nob = 1152                  #Number of bytes to send
                                #910 * 1152 = 1048320 ~(1MB)

    monitorCount = 0            #Monitor counter

    useSoftAP = 0               #Join using, 0 = Cx commands, 1 = A0 (SoftAP)
    serverEcho = 1              #Server is going to echo data back, 0=No, 1=Yes
    chgBaudRate = 0             #Enable change baud rate, 0=Disable, 1=Enable
    printData = 0               #Print send and receive data, 0=No, 1=Yes

    #Start Application
    print( "eS-WiFi TCP Client HTTP Demo:" )

    if chgBaudRate == 1:
        rcv = sendCMDBlocking(ser, "U2=" + str(bd))     #Set Baud
        ser.write("U0\r")                               #Start
        time.sleep(.1)                                  #Wait 100ms
        ser.close()                                     #Close
        ser = serial.Serial(port,bd)                    #Reopen
        rcv = sendCMDBlocking(ser, "U?")                #UART Info
        print( "Communications: " + ser.portstr + "@" + str(bd))
    else:
        print( "Communications: " + ser.portstr + "@" + str(baud))
    #print( "Packet Size:    " + str(nob) + " bytes")
    print( "" )


    #Setup data
    #data = bytearray("S3=" + str(nob) + "\r")
    #cmpData = bytearray()
    #for i in range(0, nob):
    #    data.append(255 - (i % 0x100))
    #    cmpData.append(255 - (i % 0x100))
    #End for

    
    httpDomain = "urlecho.appspot.com"
    httpURL ="GET /echo?status=200&Content-Type=text%2Fhtml&body=Hell%20world HTTP/1.1"

    #httpDomain = "www.inventeksys.com"
    #httpURL ="GET / HTTP/1.1"

    #httpDomain = "www.google.com"
    #httpURL ="GET / HTTP/1.1"

    httpHeader = "Host: "+ httpDomain + "\r\nConnection: cLose\r\n\r\n"
    httpData = ""

    #Join Network
    if (useSoftAP == 0):
        rcv = sendCMDBlocking(ser, "C1=xxxx")               #Set SSID
        rcv = sendCMDBlocking(ser, "C2=ThisIsMyNetwork1")   #Set Password
        rcv = sendCMDBlocking(ser, "C3=4")                  #WPA-2 Mixed
        rcv = sendCMDBlocking(ser, "C4=1")                  #Use DHCP

        #Get start time
        st = time.time()

        #Join
        rcv = sendCMDBlocking(ser, "C0")                        #Join network
        if (rcv != '-1'):
            if "ERROR" in rcv:                                  #Check for error
                print( "Join failed to connect to network")
                sys.exit(-1)
            else:
                print( "Time: " + ("%012.3F" % (time.time()-st)) + " Joining: " + rcv[10:] )
            #End if
        else:
            print( "Joined timed out" )
            sys.exit(-1)
        #End if
    else:
        #SoftAP
        rcv = sendCMDBlocking(ser, "Z5")                        #Get MAC address
        print( "Starting Configuration Access Point, look for Axeda_" + rcv )

        #Setup SoftAP
        rcv = sendCMDBlocking(ser, "AS=1,TCP_DEMO")             #Set SoftAP SSID

        #Start and wait for Join message
        rcv = sendCMDWait(ser, "A0","[JOIN   ]",300)            #Start SoftAP
        if (rcv != '-1'):                                       #Check for timeout
            if "ERROR" in rcv:                                  #Check for error
                print( "Join failed to connect to network")
                sys.exit(-1)
            else:
                print( "Time: " + ("%012.3F" % (time.time()-st)) + " Joining: " + rcv[10:] )
            #End if
        else:
            print( "Timeout, failed to connect to a network")
            serPort.write(ctrlQ)                                #Send Quit
            sys.exit(-1)
        #End if
    #End if

    #Configure LEDs
    rcv = sendCMDBlocking(ser, "G4=3,1")        #Red LED
    rcv = sendCMDBlocking(ser, "G4=4,1")        #Green LED

    #Green - OFF, RED - OFF
    rcv = sendCMDBlocking(ser, "G3=0,1,0")      #Red LED
    rcv = sendCMDBlocking(ser, "G3=1,1,0")      #Green LED

    #Setup and Start TCP Client
    rcv = sendCMDBlocking(ser, "P1=0")              #TCP
    rcv = sendCMDBlocking(ser, "D0=" + httpDomain)  #Remote IP
    #rcv = sendCMDBlocking(ser, "P3=192.168.1.8")     #Remote IP
    rcv = sendCMDBlocking(ser, "P4=80")           #Port
    rcv = sendCMDBlocking(ser, "P9=0")              #No certifcate verification
    rcv = sendCMDBlocking(ser, "P6=1")              #Start Client
    rcv = sendCMDBlocking(ser, "P?")              #Start Client

    #rcv = sendCMDBlocking(ser, "R2=5000")           #Read timeout

    #Get start time
    print( "Sending..." )
    st = time.time()

    #Reporting Loop
    while monitorCount < mCnt:

        data = httpURL + "\r\n" + httpHeader
        sd = "S3="+ str(len(data)) + "\r" + data
        rcv0 = sendDataBlocking(ser, sd)           #Send url + header

        dcnt = len(httpData)
        start = 0
        inc = 800
        end = start + inc
        pcnt = 1
        while dcnt > 0:
            pcnt =pcnt + 1
            sd = "S3="+ str(inc) + "\r" + httpData[start:end]
            rcv0 = sendDataBlocking(ser, sd)           #Send data
            start = end
            dcnt = dcnt - inc
            if inc > dcnt: inc=dcnt
            end = end + inc
        #Loop

        if printData == 1:
            print ("Sent Data:")
            print httpURL + "\r\n" + httpHeader + httpData
        #Endif

        pcnt = 1
        rd = ""
        if serverEcho == 1:
            while pcnt > 0:
                pcnt = pcnt - 1
                rcv1 = sendCMDBlocking(ser, "R0")        #Read echoed data
                rd = rd + rcv1
            #Loop
            if printData == 1:
                print ("Received Data:")
                print rd
                if  "200" in rd:
                    print("Passed")
                else:
                    print("Error")
                #Endif
            #Endif
        #Endif

        monitorCount += 1                       #Monitor counter
    #End while

    #Get start time
    et = time.time()
    print( "Complete" )

    #Leave Network
    rcv = sendCMDBlocking(ser, "P6=0")          #Stop Client
    rcv = sendCMDBlocking(ser, "CD")            #Leave network

    #Green - OFF, RED - OFF
    rcv = sendCMDBlocking(ser, "G3=0,1,0")      #Red LED
    rcv = sendCMDBlocking(ser, "G3=1,1,0")      #Green LED

    #Statistics
    print( "" )
    print( "Summary:" )
    print( "Total time(sec) = " + str(et-st))
    #print( "Total bytes     = " + str(monitorCount * nob))
    #print( "Total bytes/sec = " + str( (monitorCount * nob) / (et-st) ) )
    #if chgBaudRate == 1:
    #    print( "Total bits/sec =  " + str( ((monitorCount * nob) / (et-st)) * 8 ) ) + ", Theoretical = " + str( (bd/10) * 8 )
    #else:
    #    print( "Total bits/sec =  " + str( ((monitorCount * nob) / (et-st)) * 8 ) )
    #Endif

    #Reset Baud rate
    if chgBaudRate == 1:
        bd = 115200
        rcv = sendCMDBlocking(ser, "U2=" + str(bd))         #Set Baud
        ser.write("U0\r")                                   #Start
        time.sleep(.1)                                      #Wait 100ms
        ser.close()                                         #Close
        ser = serial.Serial(port,bd)                        #Reopen
        rcv = sendCMDBlocking(ser, "U?")                    #UART Info
        #print(rcv)

    #Close serial port
    ser.close()

if __name__ == '__main__':
    sys.exit(main())

